/*****************************************************************************

	Project:	ddm Hopt+Schuler - Hybrid Reader => model 862 series
	File:		Link862.h
	Author:		Marc METZINGER
	Purpose:	Driver for USI2 link with the card reader (RS232 and USB)

	Needed Software: Microsoft Visual C++ 6.0
	Needed Hardware: PC with MS-Windows 98/ME/NT/2000/XP

	Software:	V 3.6
	Date:	    Juli 2011 by hh
												(c) INES. applied informatics
 *****************************************************************************/

#define TIMEOUT_INFINITE		-1		// waiting time without timeout

////////////// Communication: USI protocol ///////////////////////////////

// USI command codes
#define usiCmd_ARM				'P'		// 0x50
#define usiCmd_ARM_DEBUG		'p'		// 0x70
#define usiCmd_ABORT			'\x1B'	// ESC

#define usiCmd_CARD_LOCK		'{'		// 0x7B
#define usiCmd_CARD_CAPTURE		'|'		// 0x7C
#define usiCmd_CARD_UNLOCK		'}'		// 0x7D

#define usiCmd_LED_GREEN_ON		'L'		// 0x4C
#define usiCmd_LED_GREEN_OFF	'l'		// 0x6C
#define usiCmd_LED_GREEN_FLASH	'('		// 0x28
#define usiCmd_LED_RED_ON		'M'		// 0x4D
#define usiCmd_LED_RED_OFF		'm'		// 0x6D
#define usiCmd_LED_RED_FLASH	')'		// 0x29

#define usiCmd_ICC_ACTIVATE		'N'		// 0x4E
#define usiCmd_ICC_DEACTIVATE	'n'		// 0x6E
#define usiCmd_ICC_WRITE		'A'		// 0x41
#define usiCmd_ICC_READ			'a'		// 0x61
#define usiCmd_ICC_SELECT		'C'		// 0x43

#define usiCmd_MEM_CARDTYPE		'T'		// 0x54
#define usiCmd_MEM_COMMAND		'B'		// 0x42

#define usiCmd_MAG_ISO_T1		'Q'		// 0x51
#define usiCmd_MAG_ISO_T2		'R'
#define usiCmd_MAG_ISO_T3		'S'
#define usiCmd_MAG_FMT_T1		'q'		// 0x71
#define usiCmd_MAG_FMT_T2		'r'
#define usiCmd_MAG_FMT_T3		's'
#define usiCmd_MAG_CUSTOM_T1	'U'		// 0x55
#define usiCmd_MAG_CUSTOM_T2	'V'
#define usiCmd_MAG_CUTSOM_T3	'W'
#define usiCmd_MAG_ERROR		'I'		// 0x49

#define usiCmd_MAINTENANCE		'!'		// 0x21
#define usiCmd_POSITION			'8'
#define usiCmd_RESET			'\x7F'	// 0x7F
#define usiCmd_RETRANSMIT		'%'		// 0x25
#define usiCmd_STATUS			'$'		// 0x24
#define usiCmd_DESCRIPTORS		'7'		// 0x37
#define usiCmd_VERSION			'9'

// extension
#define usiCmd_CHIP2_SET_CMD	'Y'		// Chip => Set/Get indirect command
#define usiCmd_CHIP2_XFER_IND	'X'		// Chip => apdu transfer
#define usiCmd_CHIP2_ORIGIN		'x'		// Chip => last indirect origin

// USI Returned response codes from CHD
#define usiRet_ACK				0x5E	// '^' Acknowledge
#define usiRet_ERROR			0x2A	// '*' Error to complete the command / Mag.decoding error
#define usiRet_INVALID			0x21	// '!' Command not recognized or not allowed
#define usiRet_MAG_DETECT_ON	0x28	// '(' Mag media is detected
#define usiRet_MAG_DETECT_OFF	0x29	// ')' Mag media detection has gone inactive
#define usiRet_NO_DATA			0x2B	// '+' No magnetic data decoded / No mag.error to be returned
#define usiRet_NO_MAG_CARD		0x3E	// '>' No magnetic tracks detected
#define usiRet_PROT_ERROR		0x3F	// '?' Protocol Communication error
#define usiRet_START			0x3A	// ':' The reader is starting (Power ON)
#define usiRet_UNAVAILABLE		0x7E	// '~' Hardware not available to complete it
// Card position codes
#define usiRet_POS_NOCARD		0x70	// 'p' No card detected
#define usiRet_POS_FRONT		0x71	// 'q' Card is detected in front
#define usiRet_POS_INSIDE		0x73	// 's' Card is seated

//////////////// READER'S STATUS WORDS /////////////////////////////

// Flags for "Status1":
#define STAT1_CARD_DETECTED		0x01	// Start switch detection
#define STAT1_CARD_INSIDE		0x02	// End switch detection
#define STAT1_LOCKED			0x04	// Card is latched
#define STAT1_ICC_ON			0x08	// ICC is powered ON
#define STAT1_SWITCH_MODE		0x10	// Automatic slot switch report
#define STAT1_MAG_DATA			0x20	// Magstripe data
#define STAT1_ARMED				0x40	// Armed to read (normal mode)
#define STAT1_ATR_MODE			0x80	// Send auto ATR

// Flags for "Status2":
#define STAT2_LED_GREEN_ON		0x01	// User leds: color Green
#define STAT2_LED_GREEN_FLASH	0x02	// User leds is flashing
#define STAT2_LED_RED_ON		0x04	// User leds: color Red
#define STAT2_LED_RED_FLASH		0x08	// User leds is flashing
#define STAT2_TEST_ARM			0x10	// Armed to read (in test mode)
#define STAT2_HEAD_ARM			0x20	// Armed to read (with pre-head detection)

// Flags for "Status3":
#define STAT3_SIM_SELECT		0x07	// mask: Current ICC selection
#define STAT3_SIM_INSIDE(i)  (0x04<<i)	// SIM presence into SAM#i, with i=1..5

// Flags of "Status4": Chip communication
#define STAT4_ELEC_ERR			0x01	// Electrical abnormality
#define STAT4_ATR_TIMEOUT		0x02	// timeout during ATR reception
#define	STAT4_RX_TIMEOUT		0x04	// reception character timeout
#define STAT4_RX_PARITY			0x08	// too many parity error during reception
#define STAT4_TX_COLLISION		0x10	// IO line is detected 'low' before transmission
#define STAT4_TX_PARITY			0x20	// too many parity error during transmission
#define STAT4_PROT_T0			0x40	// protocol T=0 error
#define STAT4_PROT_T1			0x80	// protocol T=1 error

//////////////////// LINK 862 ///////////////////////////////////////////////

// Parity code for L862_OpenSerial()
#define cPARITY_NONE			'N'
#define cPARITY_ODD				'O'
#define cPARITY_EVEN			'E'

// parameter for L862_ExecArmToReadEx()
#define usiArm_DEBUG			0	// '0': Using debug mode command
#define usiArm_WITH_PREHEAD		1	// '1': with mag stripe pre-detection

// parameter for L862_ChipActivateEx()
#define usiIcc_ONLY_MEM			'0'		// Activate only in synchronous mode
#define usiIcc_ONLY_ISO			'1'		// Only in asynchronous mode: ISO 7816-3
#define usiIcc_ONLY_EMV			'2'		// Only in asynchronous: EMV2000 - payment requirement

// parameter for L862_GetDescription()
#define usiDesc_SERIAL			'0'
#define usiDesc_COPYRIGHT		'1'
#define usiDesc_MODEL			'2'
#define usiDesc_PCB				'3'
#define usiDesc_APPLI_VERSION	'4'
#define usiDesc_BOOT_VERSION	'5'
#define usiDesc_LOADER_VERSION	'6'
#define usiDesc_CONFIG			'7'
#define usiDesc_USB_LIB			'8'

// parameter for L862_ReadIsoTrack()
#define TRACK1					1
#define TRACK2					2
#define TRACK3					3
#define ISO						0	// standard ISO format depending on selected track
#define ISO1					1
#define ISO2					2
#define ISO3					3

// CODE returned by all L862_functions()
#define L862_OK					1		// function successful performed
#define L862_NEGATIVE			0		// function unsuccessful

#define L862_NO_RES				-1		// no response returned
#define L862_UNDERRUN			-2		// not fast enough to read response
#define L862_OVERRUN			-3		// previous command is still in process

#define L862_DATA_LARGE			-10		// Command Param Size is too large
#define L862_BUFFER_SMALL		-11		// Given Buffer for Response data is too small
#define L862_ABNORMAL_RESPONSE	-12		// Returned response not conformed to resquest

#define L862_ERR_ARGUMENTS		-20		// Error into argument values at function call
#define L862_ERR_PORT_CLOSE		-21		// Access to a Port which is NOT opened
#define L862_ERR_OPEN_PORT		-22		// Unable to open this Port
#define L862_ERR_OPEN_OVERFLOW	-23		// The max limit of open Port excedded
#define L862_ERR_INSTALL		-24		// Install has to executed first
#define L862_ERR_USB_LINK		-25		// Errors returned by USB driver

//////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
extern "C" {
#endif
////////////////////////////

typedef void __stdcall T_Event862_Funct( short hPort, short Retcode );

//////////////////////////////////////////////////
// Install/open a driver with the 862 card reader
#define cL862_PORT_MAX			8

BOOL  __stdcall L862_Install( HWND hAppWnd, void *EventFct, unsigned int WM_862 );
BOOL  __stdcall L862_Free (void);

short __stdcall L862_Open( LPCSTR strPort ); // default: 38400,N,8,1
short __stdcall L862_OpenSerial( LPCSTR strPort, int Baudrate, char Parity );
short __stdcall L862_OpenUsb( short Channel );
void  __stdcall L862_Close ( short hPort );

BOOL  __stdcall L862_Abort ( short hPort );
BOOL  __stdcall L862_IsRunning( short hPort );
BOOL  __stdcall L862_GetUsbChannel( short hPort, short *Channel );

//////////////////////////////////////////////////
//	Low-level data transfer
short __stdcall L862_SendCommand  ( short hPort, short Command );
short __stdcall L862_SendCommandEx( short hPort, short Cmd, LPCSTR Param, short ParamLen );
short __stdcall L862_GetResponse  ( short hPort, short *Len, LPSTR Response, int Timeout );

//////////////////////////////////////////////////
//	High-level data transfer
short __stdcall L862_ExecReset( short hPort );
short __stdcall L862_ExecArmAbort( short hPort );
short __stdcall L862_ExecArmToRead( short hPort );
short __stdcall L862_ExecArmToReadEx( short hPort, short mode );

short __stdcall L862_ExecLocking( short hPort, bool bLock );
short __stdcall L862_ExecCapture( short hPort );
short __stdcall L862_SetGreenLed( short hPort, short State );
short __stdcall L862_SetRedLed  ( short hPort, short State );

short __stdcall L862_ReadVersion( short hPort, short *DataLen, LPSTR Data );
short __stdcall L862_ReadDescription( short hPort, short DescCode, short *DataLen, LPSTR Data );
short __stdcall L862_ReadStatus( short hPort, short *Status );
short __stdcall L862_ReadStatusEx( short hPort, short *Status1, short *Status2 );
short __stdcall L862_ReadCardPosition( short hPort, short *Position );

short __stdcall L862_ReadIsoTrack( short hPort, short nTrack, short isoFmt, short *Len, LPSTR Data );
short __stdcall L862_ReadCustomTrack( short hPort, short nTrack, short charFmt, short *Len, LPSTR Data );
short __stdcall L862_ReadCorruptedTrack( short hPort, short *Len, LPSTR Data );

short __stdcall L862_ChipSelect ( short hPort, short iSam );
short __stdcall L862_ChipActivate( short hPort, short *AtrLen, LPSTR strAtr );
short __stdcall L862_ChipActivateEx( short hPort, short Specif, short *AtrLen, LPSTR strAtr );
short __stdcall L862_ChipDeactivate( short hPort );

// For Asynchronous processor cards
short __stdcall L862_ChipTransfer( short hPort, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu );
short __stdcall L862_ChipTransferEx( short hPort, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu, int Timeout );

// Obsolete => Use "L862_ChipTransfer"
 short __stdcall L862_ChipRead ( short hPort, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu );
 short __stdcall L862_ChipWrite( short hPort, short cApduLen, LPCSTR cApdu, short *SW );

// indirect commands
short __stdcall L862_ChipSetIndirect( short hPort, short idCmd, short nRedirect, LPCSTR cApduHeader, short Le );
short __stdcall L862_ChipGetIndirect( short hPort, short idCmd, short *nRedirect, LPSTR cApduHeader, short *Le );
short __stdcall L862_ChipIndirectXfer(short hPort, short idCmd, LPCSTR Mask, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu );
short __stdcall L862_ChipIndirectXferEx(short hPort, short idCmd, LPCSTR Mask, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu, int Timeout );

// For synchronous memory card
short __stdcall L862_MemSelectType ( short hPort, short iType, short iSubType );
short __stdcall L862_MemTransfer( short hPort, short cApduLen, LPCSTR cApdu, short *rApduLen, LPSTR rApdu );

// Maintenance
short __stdcall L862_Maintenance( short hPort, LPCSTR strParam );
short __stdcall L862_MaintenanceEx( short hPort, LPCSTR strParam, short *Len, LPSTR Data, int timeout );

//////////////////////////////////////////////////
//	Status response decoding, coming from L862_ReadStatus() and L862_ReadStatusEx()

BOOL __stdcall Get862_IsCardPresent( short hPort );
BOOL __stdcall Get862_IsCardSeated( short hPort );
BOOL __stdcall Get862_IsLocked( short hPort );
BOOL __stdcall Get862_IsIccActive( short hPort, short *Sam );
BOOL __stdcall Get862_IsAutoSwitchReport( short hPort );
BOOL __stdcall Get862_IsMagnetData( short hPort );
BOOL __stdcall Get862_IsArmed( short hPort );
BOOL __stdcall Get862_IsAutoAtrMode( short hPort );
BOOL __stdcall Get862_IsIccIntoSam( short hPort, short Sam );

//////////////////////////////////////////////////
//	Library informations
LPCSTR __stdcall Get862_DllVersion(void);
LPCSTR __stdcall Get862_UsbVersion(void);
LPCSTR __stdcall Get862_ErrorText( short ErrorCode );

short  __stdcall Get862_ReturnedCode( short hPort );
LPCSTR __stdcall Get862_ReturnedText( short RetCode );

void __stdcall Get862_DllVersionB( LPSTR strText, short textLen );
void __stdcall Get862_ErrorTextB( short errCode, LPSTR strText, short textLen );
void __stdcall Get862_ReturnedTextB( short retCode, LPSTR strText, short textLen );

//////////////////////////////////////////////////////////////////////////////
#ifdef __cplusplus
}
#endif

